import json
from copy import deepcopy

from jsonobject import JsonObject, StringProperty, ListProperty, ObjectProperty, DictProperty, IntegerProperty

SAMPLE_LEX_EVENT = {
    "currentIntent": {
        "name": "intent-name",
        "slots": {
            "slot name1": "value",
            "slot name2": "value"
        },
        "slotDetails": {
            "slot name1": {
                "resolutions": [
                    {"value": "resolved value"},
                    {"value": "resolved value"}
                ],
                "originalValue": "original text"
            },
            "slot name2": {
                "resolutions": [
                    {"value": "resolved value"},
                    {"value": "resolved value"}
                ],
                "originalValue": "original text"
            }
        },
        "confirmationStatus": "None, Confirmed, or Denied (intent confirmation, if configured)"
    },
    "bot": {
        "name": "bot name",
        "alias": "bot alias",
        "version": "bot version"
    },
    "userId": "User ID specified in the POST request to Amazon Lex.",
    "inputTranscript": "Text used to process the request",
    "invocationSource": "FulfillmentCodeHook or DialogCodeHook",
    "outputDialogMode": "Text or Voice, based on ContentType request header in runtime API request",
    "messageVersion": "1.0",
    "sessionAttributes": {
        "key1": "value",
        "key2": "value"
    },
    "requestAttributes": {
        "key1": "value",
        "key2": "value"
    }
}


class PyLexObject(JsonObject):
    def to_dict(self):
        return self.to_json()


class ResolvedValueProperty(PyLexObject):
    """ {"value": "resolved value"} """
    value = StringProperty('')


class SlotDetailValueProperty(PyLexObject):
    """
    {
        "resolutions": [
            {"value": "resolved value"},
            {"value": "resolved value"}
        ],
        "originalValue": "original text"
    }
    """

    originalValue = StringProperty('')
    resolutions = ListProperty(ResolvedValueProperty)
    """ :type : List[ResolvedValueProperty]"""


class SlotsProperty(PyLexObject):
    pass


class SessionAttributesProperty(PyLexObject):
    pass


class RequestAttributesProperty(PyLexObject):
    pass


class CurrentIntentProperty(PyLexObject):
    name = StringProperty('')
    slots = ObjectProperty(SlotsProperty)
    slotDetails = DictProperty(SlotDetailValueProperty)
    confirmationStatus = StringProperty('')


class BotProperty(PyLexObject):
    name = StringProperty('')
    alias = StringProperty('')
    version = StringProperty('')


class LexInputEvent(PyLexObject):
    currentIntent = ObjectProperty(CurrentIntentProperty)
    """ :type : CurrentIntentProperty """
    bot = ObjectProperty(BotProperty)
    """ :type : BotProperty """
    userId = StringProperty('')
    inputTranscript = StringProperty('')
    invocationSource = StringProperty('')
    outputDialogMode = StringProperty('')
    messageVersion = StringProperty('')
    sessionAttributes = ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """
    requestAttributes = ObjectProperty(RequestAttributesProperty)
    """ :type : RequestAttributesProperty """

    @classmethod
    def create_class(cls, slots_property_class):
        class NewCurrentIntentProperty(CurrentIntentProperty):
            slots = ObjectProperty(slots_property_class)
            """ :type : SlotsProperty """

        class NewLexInputEvent(cls):
            currentIntent = ObjectProperty(NewCurrentIntentProperty)
            """ :type : CurrentIntentProperty """

        return NewLexInputEvent


class GenericAttachmentButtonProperty(PyLexObject):
    text = StringProperty('')
    value = StringProperty('')


class GenericAttachmentsProperty(PyLexObject):
    title = StringProperty('')
    subTitle = StringProperty('')
    imageUrl = StringProperty('')
    attachmentLinkUrl = StringProperty('')
    buttons = ListProperty(GenericAttachmentButtonProperty)
    """ :type : List[GenericAttachmentButtonProperty] """


class ResponseCardProperty(PyLexObject):
    version = IntegerProperty()
    contentType = StringProperty('application/vnd.amazonaws.card.generic')
    genericAttachments = ListProperty(GenericAttachmentsProperty)
    """ :type : List[GenericAttachmentsProperty] """


class MessageProperty(PyLexObject):
    contentType = StringProperty('PlainText')
    content = StringProperty('')


class DialogActionProperty(PyLexObject):
    type = StringProperty('')
    message = ObjectProperty(MessageProperty)
    """ :type : MessageProperty """
    responseCard = ObjectProperty(ResponseCardProperty)
    """ :type : ResponseCardProperty """


class DialogActionSlotsProperty(DialogActionProperty):
    slots = ObjectProperty(SlotsProperty)
    """ :type : dict[str, str] """


class LexOutputResponse(PyLexObject):
    dialogAction = ObjectProperty(DialogActionProperty)
    """ :type : DialogActionProperty """
    sessionAttributes = ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """

    def update_from_input(self, event):
        """

        :type event: LexInputEvent
        :return:
        """
        self.update_session_attributes(event)

    def update_session_attributes(self, event):
        for key, val in event.sessionAttributes.iteritems():
            self.sessionAttributes[key] = val

    def to_json(self):
        d = super(LexOutputResponse, self).to_json()
        if hasattr(self.dialogAction, 'message') and not self.dialogAction.message.content:
            del d['dialogAction']['message']
        if hasattr(self.dialogAction, 'responseCard') and self.dialogAction.responseCard.version is None:
            del d['dialogAction']['responseCard']
        return d


class CloseLexOutputResponse(LexOutputResponse):
    class CloseDialogActionProperty(DialogActionProperty):
        type = StringProperty('Close')
        fulfillmentState = StringProperty('Fulfilled')

    dialogAction = ObjectProperty(CloseDialogActionProperty)
    """ :type : CloseLexOutputResponse.CloseDialogActionProperty """


class LexOutputSlotsResponse(LexOutputResponse):
    class SubDialogActionSlotsProperty(DialogActionSlotsProperty):
        pass

    dialogAction = ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : LexOutputSlotsResponse.SubDialogActionSlotsProperty """

    @classmethod
    def create_class(cls, slots_property_class):

        class NewIntentOutputResponse(LexOutputSlotsResponse):
            class SubDialogActionSlotsProperty(cls.SubDialogActionSlotsProperty):
                slots = ObjectProperty(slots_property_class)

            dialogAction = ObjectProperty(SubDialogActionSlotsProperty)
            """ :type : NewIntentOutputResponse.SubDialogActionSlotsProperty """

        return NewIntentOutputResponse

    def update_from_input(self, event):
        """

        :type event: LexInputEvent
        :return:
        """
        super(LexOutputSlotsResponse, self).update_from_input(event)
        self.update_slots(event)
        self.update_intent_name(event)

    def update_intent_name(self, event):
        if hasattr(self.dialogAction, 'intentName'):
            self.dialogAction.intentName = event.currentIntent.name

    def update_slots(self, event):
        """
        :type lex_input_event: LexInputEvent
        :return: None
        """
        if isinstance(event, LexInputEvent):
            event_slots = event.currentIntent.slots
        elif isinstance(event, basestring) or isinstance(event, unicode) or isinstance(event, str):
            event_slots = deepcopy(json.loads(event)['currentIntent']['slots'])
        else:
            event_slots = deepcopy(event['currentIntent']['slots'])

        for key, val in event_slots.iteritems():
            self.dialogAction.slots[key] = val


class ConfirmIntentOutputResponse(LexOutputSlotsResponse):
    class SubDialogActionSlotsProperty(DialogActionSlotsProperty):
        type = StringProperty('ConfirmIntent')
        intentName = StringProperty('')

    dialogAction = ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : ConfirmIntentOutputResponse.SubDialogActionSlotsProperty """


class DelegateIntentOutputResponse(LexOutputSlotsResponse):
    class SubDialogActionSlotsProperty(DialogActionSlotsProperty):
        type = StringProperty('DelegateIntent')

    dialogAction = ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : DelegateIntentOutputResponse.SubDialogActionSlotsProperty """


class ElicitIntentOutputResponse(LexOutputResponse):
    class ElicitIntentDialogActionProperty(DialogActionProperty):
        type = StringProperty('ElicitIntent')

    dialogAction = ObjectProperty(ElicitIntentDialogActionProperty)
    """ :type : ElicitIntentOutputResponse.ElicitIntentDialogActionProperty """


class ElicitSlotOutputResponse(LexOutputSlotsResponse):
    class SubDialogActionSlotsProperty(DialogActionSlotsProperty):
        type = StringProperty('ElicitIntent')
        intentName = StringProperty('')
        slotToElicit = StringProperty('')

    dialogAction = ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : ElicitSlotOutputResponse.SubDialogActionSlotsProperty """



import os
from argparse import ArgumentParser

import stub_generator

def main():
    parser = ArgumentParser()
    parser.add_argument('-s','--slots', nargs='+', help='<Required> Slots names', required=True)
    parser.add_argument('-n', '--sessions', nargs='+', help='Session names', required=False)
    parser.add_argument('-f','--filepath', help='filepath', required=False)
    args = parser.parse_args()
    slots = args.slots
    sessions = args.sessions
    filepath = args.filepath

    if filepath:
        stub_generator.generate_file(filepath, slots, sessions)
    else:
        print(stub_generator.generate_file_string(slots, sessions))

if __name__ == '__main__':
    main()


from setuptools import setup, find_packages

setup(name='pylexo',
      version='0.2.0',
      packages=find_packages(),
      description='Pythonic Lex Object (PyLexO)',
      author='WavyCloud',
      author_email='',
      entry_points={
          'console_scripts': [
              'pylexo = pylexo.__main__:main'
          ]
      },
      url='https://github.com/wavycloud/pylexo',
      py_modules=['pylexo'],
      install_requires=['jsonobject==0.7.1'],
      license='MIT License',
      zip_safe=True,
      keywords='aws python lex lambda',
      classifiers=[])



import os

template = '''import pylexo


class Slots:
{slot_strings}


class SessionAttributes:
{session_strings}

class SlotsProperty(pylexo.SlotsProperty):
{slot_lines}


class SessionAttributesProperty(pylexo.SessionAttributesProperty):
{session_lines}


class CurrentIntentProperty(pylexo.CurrentIntentProperty):
    slots = pylexo.ObjectProperty(SlotsProperty)
    """ :type : SlotsProperty """


class LexInputEvent(pylexo.LexInputEvent):
    currentIntent = pylexo.ObjectProperty(CurrentIntentProperty)
    """ :type : CurrentIntentProperty """
    sessionAttributes = pylexo.ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """


class CloseLexOutputResponse(pylexo.CloseLexOutputResponse):
    sessionAttributes = pylexo.ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """


class ElicitIntentOutputResponse(pylexo.ElicitIntentOutputResponse):
    sessionAttributes = pylexo.ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """


class ElicitSlotOutputResponse(pylexo.ElicitSlotOutputResponse):
    class SubDialogActionSlotsProperty(pylexo.ElicitSlotOutputResponse.SubDialogActionSlotsProperty):
        slots = pylexo.ObjectProperty(SlotsProperty)
        """ :type : SlotsProperty """

    dialogAction = pylexo.ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : ElicitSlotOutputResponse.SubDialogActionSlotsProperty """
    sessionAttributes = pylexo.ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """


class ConfirmIntentOutputResponse(pylexo.ConfirmIntentOutputResponse):
    class SubDialogActionSlotsProperty(pylexo.ConfirmIntentOutputResponse.SubDialogActionSlotsProperty):
        slots = pylexo.ObjectProperty(SlotsProperty)
        """ :type : SlotsProperty """

    dialogAction = pylexo.ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : ConfirmIntentOutputResponse.SubDialogActionSlotsProperty """
    sessionAttributes = pylexo.ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """


class DelegateIntentOutputResponse(pylexo.DelegateIntentOutputResponse):
    class SubDialogActionSlotsProperty(pylexo.DelegateIntentOutputResponse.SubDialogActionSlotsProperty):
        slots = pylexo.ObjectProperty(SlotsProperty)
        """ :type : SlotsProperty """

    dialogAction = pylexo.ObjectProperty(SubDialogActionSlotsProperty)
    """ :type : DelegateIntentOutputResponse.SubDialogActionSlotsProperty """
    sessionAttributes = pylexo.ObjectProperty(SessionAttributesProperty)
    """ :type : SessionAttributesProperty """
'''


def generate_file_string(slots, sessions=None):
    sessions = sessions or []
    session_lines = '\n'.join(
        ['    {} = pylexo.StringProperty()'.format(session_name) for session_name in sessions]) or '    pass'
    session_strings = '\n'.join(
        ['    {} = "{}"'.format(session_name, session_name) for session_name in sessions]) or '    pass'
    slot_lines = '\n'.join(['    {} = pylexo.StringProperty()'.format(slot_name) for slot_name in slots])
    slot_strings = '\n'.join(['    {} = "{}"'.format(slot_name, slot_name) for slot_name in slots])
    file_string = template.format(slot_lines=slot_lines, slot_strings=slot_strings, session_lines=session_lines,
                                  session_strings=session_strings)
    return file_string


def generate_file(filepath, slots, sessions=None):
    file_string = generate_file_string(slots, sessions)
    with open(filepath, 'w+') as fp:
        fp.write(file_string)

